#!/bin/bash 
#
INPUTDIR="$PWD/input"
SOURCEDIR="$INPUTDIR"
#Please use full path to the mpi application
MPIAPP_LONG="/xdata/sherman/bin/mpi_cosy.gnu01.large"
MPIAPP_OPTS="phad_simo"
#Comment out if do not need notification
#EMAIL2NOTIFY="serguei@nicadd.niu.edu"
#
##Do not edit beyond this point EXCEPT to change condor job requirements
#====
if [ ! -f $MPIAPP_LONG ]; then 
  echo " Can not find  \$MPIAPP_LONG = $MPIAPP_LONG ... please check "
exit 0;
fi 
#
NPROC=1;
if [ "0$1" == "0" ]||[ "0$2" == "0" ]; then 
echo "Usage:: .$0 short_node_name  nproc_to_use ... please resubmit" 
exit 0;
fi;
#
NODE="$1"
WNODE="$NODE.nicadd.niu.edu";
NPROC=$2;
echo "Will use $NPROC MPI cores on node $WNODE"
#
DATE="$(date | sed 's/:/ /g' | awk '{print $2$3"_"$4_$5_$6}')"
MPIAPP_SHORT="$(echo $MPIAPP_LONG | sed 's/\// /g' | awk '{print $NF}')"
JOBNAME="mpi_${MPIAPP_SHORT}_${NODE}n${NPROC}"
OUTPUTDIR="$PWD/condor_out/$JOBNAME_out"
#Better have RUN_DIR on scratch
#
#For debug only 
#RUN_DIR="$PWD/condor_run/${JOBNAME}_$DATE"

JOBOUTDIR="$OUTPUTDIR/${JOBNAME}_$DATE"
LOGDIR="$JOBOUTDIR/log"
#
SCRIPT_NAME="run_$JOBNAME"
RUNSCRIPT="${SCRIPT_NAME}_${DATE}.sh"
CMDSCRIPT="${SCRIPT_NAME}_${DATE}.cmd"
#
if [ ! -d $LOGDIR ]; then
 mkdir -p $LOGDIR
fi
#
if [ ! -d $OUTPUTDIR/$JOBOUTDIR ]; then
 mkdir -p $JOBOUTDIR
fi
#Write script to run on a remote node
if [ -f $RUNSCRIPT ]; then
   /bin/rm -f $RUNSCRIPT 
fi
cat > $RUNSCRIPT  <<EOF
#!/bin/bash
###**************************************************************
##!!!For debug
##set -x
#
#This is necessary to define the modules environment
. /usr/share/Modules/init/bash

# Set $MPI_HOME and include mpi application in the PATH
#module load impactT/impactTv2.0-openmpi-1.8.1
module load openmpi/openmpi-1.8.1
#
_CONDOR_PROCNO=\$_CONDOR_PROCNO
_CONDOR_NPROCS=\$_CONDOR_NPROCS

CONDOR_SSH=\`condor_config_val libexec\`
CONDOR_SSH=\$CONDOR_SSH/condor_ssh

SSHD_SH=\`condor_config_val libexec\`
SSHD_SH=\$SSHD_SH/sshd.sh

env | grep CONDOR
echo . \$SSHD_SH \$_CONDOR_PROCNO \$_CONDOR_NPROCS 
. \$SSHD_SH \$_CONDOR_PROCNO \$_CONDOR_NPROCS 

#
# If not the head node, just sleep forever, to let the
# sshds run
if [ \$_CONDOR_PROCNO -ne 0 ]
then
		wait
		sshd_cleanup
		exit 0
fi
###
#We are on the remote parent node
HOSTNAME_SHORT="\$(hostname -s)"
RUN_DIR="/nfs/work/\${HOSTNAME_SHORT}/$USER/${JOBNAME}_$DATE"
#
echo  "User \$USER at  NODE=\$HOSTNAME_SHORT"
echo  "Run_dir = \$RUN_DIR"
#echo "PATH= \$PATH "
#echo "LD_LIBRARY_PATH= \$LD_LIBRARY_PATH "

mkdir -p  \$RUN_DIR
rsync -av $INPUTDIR/* \$RUN_DIR/
#Create source dir (in case we need compile code locally)
#mkdir -p \$RUN_DIR/src
#rsync -av $SOURCEDIR/* \$RUN_DIR/src
#
rsync -av \$_CONDOR_SCRATCH_DIR/contact \$RUN_DIR/
#
#CONDOR_CONTACT_FILE=\$_CONDOR_SCRATCH_DIR/contact
CONDOR_CONTACT_FILE=\$RUN_DIR/contact
export CONDOR_CONTACT_FILE
#
#
cd \$RUN_DIR
ls -ltrh
#
# The second field in the contact file is the machine name
# that condor_ssh knows how to use

echo "sort -n -k 1 < \$CONDOR_CONTACT_FILE | awk '{print \$2}'" 
sort -n -k 1 < \$CONDOR_CONTACT_FILE | awk '{print \$2}' > machines
ls -l  machines
cat machines
#Check memlock limits (mpi likes unlimited)
echo "Condor ulimits (as set in /usr/lib/systemd/system/condor.service)"
ulimit -a
#
## run the actual mpi  for NAMD
echo "#---------------------------"
echo "Running ${MPIAPP_LONG}"
#
MPIAPP=${MPIAPP_LONG}
#
sleep 2
echo   mpirun -v --prefix \$MPI_HOME  -n \$_CONDOR_NPROCS -hostfile machines  ${MPIAPP_LONG}  ${MPIAPP_OPTS}
       mpirun -v --prefix \$MPI_HOME  -n \$_CONDOR_NPROCS -hostfile machines  ${MPIAPP_LONG}  ${MPIAPP_OPTS}

#echo   mpirun -v --prefix \$MPI_HOME --mca plm_rsh_agent \$CONDOR_SSH -n \$_CONDOR_NPROCS -hostfile machines \$NAMD \$NAMD_OPTS  \$NAMD_CONFIG
#       mpirun -v --prefix \$MPI_HOME --mca plm_rsh_agent \$CONDOR_SSH -n \$_CONDOR_NPROCS -hostfile machines \$NAMD \$NAMD_OPTS  \$NAMD_CONFIG

echo "Run finished"
rsync -av * $JOBOUTDIR/

sshd_cleanup


/bin/ls -l  \$RUN_DIR
#Be careful with  this (the scratch area should be on local /disk) 
cd /disk
/bin/rm -rf $USER/${JOBNAME}_$DATE
#
exit $?
EOF
#
chmod 755 $RUNSCRIPT 
#Prepare condor command file
if [ "0$EMAIL2NOTIFY" != "0" ]; then 
NOTIFY_CMD="notify_user = \$EMAIL2NOTIFY"
fi
cat > $CMDSCRIPT  <<EOF
##################################################
##Condor command file parallel universe
#
universe      = parallel
#number of mpi processes
machine_count = $NPROC
#
requirements = (Arch == "SL7x64")&&((Machine == "$WNODE"))
#requirements = (Arch == "SL7x64")&&((Machine == "phpc0.nicadd.niu.edu")||(Machine == "phpc1.nicadd.niu.edu")||(Machine == "phpc2.nicadd.niu.edu") )
executable                = $RUNSCRIPT 
output                    = $LOGDIR/${DATE}-${SCRIPT_NAME}.out
error                     = $LOGDIR/${DATE}-${SCRIPT_NAME}.err
log                       = $LOGDIR/${DATE}-${SCRIPT_NAME}.log
#
getenv                    = true
should_transfer_files     = yes
when_to_transfer_output   = on_exit_or_evict
notification              = always
#Send e-mail to $EMAIL2NOTIFY 
$NOTIFY_CMD
queue
EOF
#

#
#save $CMDSCRIPT and $RUNSCRIPT 
/bin/cp -rfvp $CMDSCRIPT $JOBOUTDIR/
/bin/cp -rfvp $RUNSCRIPT $JOBOUTDIR/

echo "Submitting job:: condor_submit  $CMDSCRIPT"
sleep 5
condor_submit $CMDSCRIPT
echo "Job stat will be written to $LOGDIR"
echo "To check job status please run :"
echo "condor_q -global" 
echo "#------------#"
#

